@@ -83,7 +83,7 @@ short ospeed;
 
 struct display *display, *displays;
 #ifdef COLOR
-int  attr2color[8];
+int  attr2color[8][4];
 int  nattr2color;
 #endif
 
@@ -833,6 +833,13 @@ int c;
 	  return;
 	}
 #  endif
+      if (c < 32)
+	{
+	  AddCStr2(D_CS0, '0');
+	  AddChar(c + 0x5f);
+	  AddCStr(D_CE0);
+	  goto addedutf8;
+	}
       AddUtf8(c);
       goto addedutf8;
     }
@@ -1021,12 +1028,19 @@ int mode;
 {
   if (display && D_mouse != mode)
     {
+      char mousebuf[20];
       if (!D_CXT)
 	return;
       if (D_mouse)
-        AddStr(D_mouse == 9 ? "\033[?9l" : "\033[?1000l");
+        {
+          sprintf(mousebuf, "\033[?%dl", D_mouse);
+          AddStr(mousebuf);
+	}
       if (mode)
-        AddStr(mode == 9 ? "\033[?9h" : "\033[?1000h");
+        {
+          sprintf(mousebuf, "\033[?%dl", mode);
+          AddStr(mousebuf);
+	}
       D_mouse = mode;
     }
 }
@@ -1907,8 +1921,17 @@ int f, b;
   of = rend_getfg(&D_rend);
   ob = rend_getbg(&D_rend);
 
+#ifdef COLORS16
+  /* intense default not invented yet */
+  if (f == 0x100)
+    f = 0;
+  if (b == 0x100)
+    b = 0;
+#endif
   debug2("SetColor %d %d", coli2e(of), coli2e(ob));
   debug2(" -> %d %d\n", coli2e(f), coli2e(b));
+  debug2("(%d %d", of, ob);
+  debug2(" -> %d %d)\n", f, b);
 
   if (!D_CAX && D_hascolor && ((f == 0 && f != of) || (b == 0 && b != ob)))
     {
@@ -2023,7 +2046,16 @@ struct mchar *mc;
       mmc = *mc;
       for (i = 0; i < 8; i++)
 	if (attr2color[i] && (mc->attr & (1 << i)) != 0)
-          ApplyAttrColor(attr2color[i], &mmc);
+	  {
+	    if (mc->color == 0 && attr2color[i][3])
+              ApplyAttrColor(attr2color[i][3], &mmc);
+	    else if ((mc->color & 0x0f) == 0 && attr2color[i][2])
+              ApplyAttrColor(attr2color[i][2], &mmc);
+	    else if ((mc->color & 0xf0) == 0 && attr2color[i][1])
+              ApplyAttrColor(attr2color[i][1], &mmc);
+	    else
+              ApplyAttrColor(attr2color[i][0], &mmc);
+	  }
       mc = &mmc;
       debug2("SetRendition: mapped to %02x %02x\n", (unsigned char)mc->attr, 0x99 - (unsigned char)mc->color);
     }
@@ -2352,7 +2384,7 @@ RefreshHStatus()
   evdeq(&D_hstatusev);
   if (D_status == STATUS_ON_HS)
     return;
-  buf = MakeWinMsgEv(hstatusstring, D_fore, '%', (D_HS && D_has_hstatus == HSTATUS_HS && D_WS > 0) ? D_WS : D_width - !D_CLP, &D_hstatusev);
+  buf = MakeWinMsgEv(hstatusstring, D_fore, '%', (D_HS && D_has_hstatus == HSTATUS_HS && D_WS > 0) ? D_WS : D_width - !D_CLP, &D_hstatusev, 0);
   if (buf && *buf)
     {
       ShowHStatus(buf);
@@ -2518,7 +2550,7 @@ int y, from, to, isblank;
     }
 
   p = Layer2Window(cv->c_layer);
-  buf = MakeWinMsgEv(captionstring, p, '%', D_width - !D_CLP, &cv->c_captev);
+  buf = MakeWinMsgEv(captionstring, p, '%', D_width - !D_CLP, &cv->c_captev, 0);
   if (cv->c_captev.timeout.tv_sec)
     evenq(&cv->c_captev);
   xx = strlen(buf);
@@ -2848,8 +2880,8 @@ int ins;
 	  y--;
 	}
     }
-  else if (y == D_bot)
-    ChangeScrollRegion(ys, ye);		/* remove unusable region */
+  else if (y == D_bot)	/* remove unusable region? */
+    ChangeScrollRegion(0, D_height - 1);
   if (D_x != D_width || D_y != y)
     {
       if (D_CLP && y >= 0)		/* don't even try if !LP */
