@@ -286,7 +286,7 @@ char *pt;
 	      c |= cf << 8;
 	      if (c == UCS_HIDDEN)
 		continue;
-	      c = ToUtf8(pt, c);
+	      c = ToUtf8_comb(pt, c);
 	      l += c;
 	      if (pt)
 	        pt += c;
@@ -1163,6 +1163,10 @@ int isblank;
 
   if (markdata->second == 0)
     {
+      if (dw_right(ml, xs, fore->w_encoding) && xs > 0)
+	xs--;
+      if (dw_left(ml, xe, fore->w_encoding) && xe < fore->w_width - 1)
+	xe++;
       if (xs == 0 && y > 0 && wy > 0 && WIN(wy - 1)->image[flayer->l_width] == 0)
         LCDisplayLineWrap(flayer, ml, y, xs, xe, isblank);
       else
