@@ -182,7 +182,11 @@ sigret_t (*func) __P(SIGPROTOARG);
   struct sigaction osa, sa;
   sa.sa_handler = func;
   (void)sigemptyset(&sa.sa_mask);
+#ifdef SA_RESTART
+  sa.sa_flags = (sig == SIGCHLD ? SA_RESTART : 0);
+#else
   sa.sa_flags = 0;
+#endif
   if (sigaction(sig, &sa, &osa))
     return (sigret_t (*)__P(SIGPROTOARG))-1;
   return osa.sa_handler;
