@@ -499,11 +499,19 @@
  * 05.09.2002  3.09.13 added missing break statement that broke
  *                     the eval command.
  *                     -- DISTRIBUTED
+ * 13.03.2003  3.09.15 Console workaround for linux. Scrolling-region
+ *                     reset fix. GBK encoding added.
+ *                     support for unicode combining characters added.
+ *                     openpty() support added (thomas@xs4all.nl).
+ *                     preselect of blank window ('-') or window
+ *                     list ('=') added. Added %` string escape
+ *                     and backtick command.
+ *                     -- DISTRIBUTED
  */
 
 #define ORIGIN "FAU"
 #define REV 3
 #define VERS 9
-#define PATCHLEVEL 13
-#define DATE "5-Sep-02"
-#define STATE "" 
+#define PATCHLEVEL 15
+#define DATE "13-Mar-03"
+#define STATE ""
