@@ -1020,8 +1020,16 @@ int wi, he, hi;
     {
       ml = OLDWIN(fy);
       ASSERT(ml->image);
-      for (l = 0; l < p->w_width; l++)
-      ASSERT((unsigned char)ml->image[l] >= ' ');
+# ifdef UTF8
+      if (p->w_encoding == UTF8)
+	{
+	  for (l = 0; l < p->w_width; l++)
+	    ASSERT(ml->image[l] >= ' ' || ml->font[l]);
+	}
+      else
+#endif
+        for (l = 0; l < p->w_width; l++)
+          ASSERT(ml->image[l] >= ' ');
     }
 #endif
   return 0;
