@@ -56,15 +56,12 @@ extern int log_flush, logtstamp_after;
 extern int ZombieKey_destroy, ZombieKey_resurrect;
 extern struct layer *flayer;
 extern int maxusercount;
+extern int pty_preopen;
 
 #if defined(TIOCSWINSZ) || defined(TIOCGWINSZ)
 extern struct winsize glwz;
 #endif
 
-#ifdef _IBMR2
-extern int aixhack;
-#endif
-
 #ifdef O_NOCTTY
 extern int separate_sids;
 #endif
@@ -1072,6 +1069,7 @@ char **namep;
   if (*typep != W_TYPE_PTY)
     return f;
 
+#ifndef PTYROFS
 #ifdef PTYGROUP
   if (chown(*namep, real_uid, PTYGROUP) && !eff_uid)
 #else
@@ -1092,6 +1090,7 @@ char **namep;
       close(f);
       return -1;
     }
+#endif
   return f;
 }
 
@@ -1122,23 +1121,18 @@ char **args, *ttyn;
   int i, pat, wfdused;
   struct pseudowin *pwin = win->w_pwin;
 #endif
-#if (defined(sun) || defined(_IBMR2)) && defined(O_NOCTTY)
-  int slave;
-#endif
+  int slave = -1;
 
-#if defined(sun) && defined(O_NOCTTY)
-  /* sun's utmp_update program opens the salve side, thus corrupting
-   * pty semantics */
-  debug("pre-opening slave...\n");
-  if ((slave = open(ttyn, O_RDWR|O_NOCTTY)) == -1)
+#ifdef O_NOCTTY
+  if (pty_preopen)
     {
-      Msg(errno, "ttyn");
-      return -1;
+      debug("pre-opening slave...\n");
+      if ((slave = open(ttyn, O_RDWR|O_NOCTTY)) == -1)
+	{
+	  Msg(errno, "ttyn");
+	  return -1;
+	}
     }
-#endif
-#if defined(_IBMR2) && defined(O_NOCTTY)
-  slave = aixhack;
-  aixhack = -1;
 #endif
   debug("forking...\n");
   proc = *args;
@@ -1192,15 +1186,16 @@ char **args, *ttyn;
       if (dfp && dfp != stderr)
 	fclose(dfp);
 #endif
-#if (defined(sun) || defined(_IBMR2)) && defined(O_NOCTTY)
-      close(0);
-      dup(slave);
-      close(slave);
-      closeallfiles(win->w_ptyfd);
-      slave = dup(0);
-#else
-      closeallfiles(win->w_ptyfd);
-#endif
+      if (slave != -1)
+	{
+	  close(0);
+	  dup(slave);
+	  close(slave);
+	  closeallfiles(win->w_ptyfd);
+	  slave = dup(0);
+	}
+      else
+        closeallfiles(win->w_ptyfd);
 #ifdef DEBUG
       if (dfp)	/* do not produce child debug, when debug is "off" */
 	{
@@ -1277,9 +1272,8 @@ char **args, *ttyn;
       dup(0);
 #endif /* PSEUDOS */
       close(win->w_ptyfd);
-#if (defined(sun) || defined(_IBMR2)) && defined(O_NOCTTY)
-      close(slave);
-#endif
+      if (slave != -1)
+        close(slave);
       if (newfd >= 0)
 	{
 	  struct mode fakemode, *modep;
@@ -1378,9 +1372,8 @@ char **args, *ttyn;
     default:
       break;
     }
-#if (defined(sun) || defined(_IBMR2)) && defined(O_NOCTTY)
-  close(slave);
-#endif
+  if (slave != -1)
+    close(slave);
   return pid;
 }
 
