@@ -168,6 +168,9 @@ struct win
 #ifdef FONT
   char	 w_FontL;		/* character font GL */
   char	 w_FontR;		/* character font GR */
+# ifdef ENCODINGS
+  char	 w_FontE;		/* character font GR locked */
+# endif
   int	 w_Charset;		/* charset number GL */
   int	 w_CharsetR;		/* charset number GR */
   int	 w_charsets[4];		/* Font = charsets[Charset] */
@@ -255,6 +258,15 @@ struct win
   char   w_telsubbuf[128];
   int    w_telsubidx;
   struct event w_telconnev;
+#endif
+  struct mline *w_alt_mlines;
+  int    w_alt_width;
+  int    w_alt_height;
+  int    w_alt_histheight;
+  int    w_alt_x, w_alt_y;
+#ifdef COPY_PASTE
+  struct mline *w_alt_hlines;
+  int    w_alt_histidx;
 #endif
 };
 
